#ifndef __SMS_DATAEXPLORER__
#define __SMS_DATAEXPLORER__

#include <CLAM/Slotv2.hxx>
#include <CLAM/Slotv1.hxx>
#include <CLAM/Slotv0.hxx>
#include <CLAM/Signalv1.hxx>
#include <CLAM/DataTypes.hxx>
#include <CLAM/LogMagSpectrumAdapter.hxx>
#include <CLAM/AudioAdapter.hxx>
#include <CLAM/SinTracksAdapter.hxx>
#include <CLAM/SpectralPeakArrayAdapter.hxx>
#include <CLAM/FundamentalAdapter.hxx>

#include <FL/Fl_Widget.H>

class Fl_Smart_Tile;

namespace CLAM
{
	class Audio;
	class Frame;
	class Segment;
}

namespace CLAMVM
{
	class Fl_SMS_Browsable_Playable_Audio;
	class Fl_SMS_Spectrum;
	class Fl_SMS_SinTracks_Browser;
	class Fl_SMS_SpectrumAndPeaks;
	class Fl_SMS_FundFreq_Browser;

	class SMS_DataExplorer
	{
	public:
		
		SMS_DataExplorer();
		~SMS_DataExplorer();

		SigSlot::Slotv1< const CLAM::Segment& >   NewSegment;
		SigSlot::Slotv2< const CLAM::Frame&, bool >     NewFrame;
		SigSlot::Slotv1< const CLAM::Audio& >     NewInputAudio;
		SigSlot::Slotv1< const CLAM::Audio& >     NewSynthesizedAudio;
		SigSlot::Slotv1< const CLAM::Audio& >     NewSynthesizedSinusoidal;
		SigSlot::Slotv1< const CLAM::Audio& >     NewSynthesizedResidual;
		SigSlot::Signalv1< double >               SelectedTime;
		SigSlot::Slotv2< CLAM::TData, CLAM::TData> SetFundFreqRangeHint;

		SigSlot::Slotv0                           ShowInputAudio;
		SigSlot::Slotv0                           ShowSinTracks;
		SigSlot::Slotv0                           ShowFundFreq;
		SigSlot::Slotv0                           ShowSpectrumAndPeaks;
		SigSlot::Slotv0                           ShowSinusoidalSpectrum;
		SigSlot::Slotv0                           ShowResidualSpectrum;
		SigSlot::Slotv0                           ShowSynthesizedAudio;
		SigSlot::Slotv0                           ShowSynthesizedSinusoidal;
		SigSlot::Slotv0                           ShowSynthesizedResidual;
		

		void SetCanvas( Fl_Smart_Tile* pCanvas );
		void CloseAll();

	protected:
		
		// "Proxy" Slot that is always connected to SelectedFrame signal 
		SigSlot::Slotv1< double >                 SelectedTimeChanged; 

		void OnNewSegment( const CLAM::Segment& );
		void OnNewFrame( const CLAM::Frame& , bool fullAnalysisData );
		void OnNewInputAudio( const CLAM::Audio& );
		void OnNewSynthesizedAudio( const CLAM::Audio& );
		void OnNewSynthesizedSinusoidal( const CLAM::Audio& );
		void OnNewSynthesizedResidual( const CLAM::Audio& );
		void OnSelectedTimeChanged( double time  );
		void OnNewFundFreqRangeHint( CLAM::TData lower, CLAM::TData higher );

		void OnShowInputAudio();
		void OnShowSinTracks();
		void OnShowFundFreq();
		void OnShowSpectrumAndPeaks();
		void OnShowSinusoidalSpectrum();
		void OnShowResidualSpectrum();
		void OnShowSynthesizedAudio();
		void OnShowSynthesizedSinusoidal();
		void OnShowSynthesizedResidual();

		// Smart Tile detach callback methods
		void Detach();
		static void sDetachCb( Fl_Widget*, void* );

	private:
		
		// Adapters
		LogMagSpectrumAdapter        mSpectrumAdapter;
		LogMagSpectrumAdapter        mSinusoidalSpectrumAdapter;
		LogMagSpectrumAdapter        mResidualSpectrumAdapter;

		SinTracksAdapter             mSinusoidalTracksAdapter;
		
		FundamentalAdapter           mFundFreqAdapter;

		SpectralPeakArrayAdapter     mPeakArrayAdapter;

		AudioAdapter                 mOriginalAudioAdapter;
		AudioAdapter                 mSynthesizedAudioAdapter;
		AudioAdapter                 mSynthesizedSinusoidalAdapter;
		AudioAdapter                 mSynthesizedResidualAdapter;

		// Widgets
		Fl_SMS_Browsable_Playable_Audio*            mpOriginalAudioWidget;
		Fl_SMS_Browsable_Playable_Audio*            mpSynthesizedAudioWidget;
		Fl_SMS_Browsable_Playable_Audio*            mpSynthesizedResidualWidget;
		Fl_SMS_Browsable_Playable_Audio*            mpSynthesizedSinusoidalWidget;
		Fl_SMS_SpectrumAndPeaks*         mpSpectrumAndPeaksWidget;
		Fl_SMS_Spectrum*                 mpSinusoidalSpectrum;
		Fl_SMS_Spectrum*                 mpResidualSpectrum;
		Fl_SMS_SinTracks_Browser*        mpSegmentSinTracks;
		Fl_SMS_FundFreq_Browser*         mpFundFreqBrowser;
		Fl_Smart_Tile*                   mpCanvas;

		double                           mCurrentFrameCenterTime;
		CLAM::TData                      mHigherF0Hint;
		CLAM::TData                      mLowerF0Hint;
	};

}

#endif // SMS_DataExplorer.hxx
